package org.jboss.qa.drools.expirescloud;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.EventProcessingOption;
import org.drools.core.util.debug.SessionInspector;
import org.drools.core.util.debug.SessionReporter;
import org.drools.definition.type.FactType;
import org.drools.io.ResourceFactory;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.WorkingMemoryEntryPoint;

public class ExpiresTest {
    private static final String pkg = "org.jboss.qa.drools.cloud";
    private static final String event1 = "Event";
    private static final String entryPoint = "entryPoint";

    public static void main(String args[]) throws InstantiationException, IllegalAccessException {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(ResourceFactory.newInputStreamResource(ExpiresTest.class.getResourceAsStream("expirescloud.drl")), ResourceType.DRL);

        if (kbuilder.hasErrors()) {
            for (KnowledgeBuilderError e : kbuilder.getErrors()) {
                System.err.println(e);
            }
            System.exit(1);
        }

        KnowledgeBaseConfiguration kbconf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
        kbconf.setOption(EventProcessingOption.CLOUD);

        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(kbconf);
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

        KnowledgeSessionConfiguration ksconf = KnowledgeBaseFactory.newKnowledgeSessionConfiguration();

        StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession(ksconf, null);
        WorkingMemoryEntryPoint ep = ksession.getWorkingMemoryEntryPoint(entryPoint);
        SessionInspector inspector = new SessionInspector(ksession);

        ep.insert(new TestEvent("Darth Vader", "Death Star"));
        ksession.fireAllRules();

        FactType event = kbase.getFactType(pkg, event1);
        Object e1 = event.newInstance();
        event.set(e1, "name", "someKey");
        event.set(e1, "value", "someValue");

        ep.insert(e1);
        ksession.fireAllRules();

        ksession.dispose();
        ksession = kbase.newStatefulKnowledgeSession();
        ep = ksession.getWorkingMemoryEntryPoint(entryPoint);

        ep.insert(new TestEvent("Darth Sidious", "Coruscant"));
        ksession.fireAllRules();

        System.out.println(SessionReporter.generateReport("simple", inspector.getSessionInfo(), null));
    }

}
